<?php

/**
 * 百度空间文章 -> Wordpress 导入工具
 * 
 * @author Hessian <hess.4x@gmail.com>
 */

require( dirname(__FILE__) . '/../wp-load.php' );
require( dirname(__FILE__) . '/../wp-admin/includes/taxonomy.php' );

class BaiduHiToWrodpress
{
  protected $baiduAccount;
  protected $cookie;

  protected $imageUrlProcessor;

  public function __construct( $account, $cookie = '' )
  {
    $this->baiduAccount = $account;
    $this->cookie = $cookie;
    
    $importer = $this;
    
    $this->imageUrlProcessor = function( $matches ) use ( $importer )
    {
      /**
      $upload_dir = Array ( 
        [path] => C:\path\to\wordpress\wp-content\uploads\2010\05 
        [url] => http://example.com/wp-content/uploads/2010/05 
        [subdir] => /2010/05 
        [basedir] => C:\path\to\wordpress\wp-content\uploads 
        [baseurl] => http://example.com/wp-content/uploads 
        [error] => 
      ) 
      // Descriptions
      [path] - base directory and sub directory or full path to upload directory.
      [url] - base url and sub directory or absolute URL to upload directory.
      [subdir] - sub directory if uploads use year/month folders option is on.
      [basedir] - path without subdir.
      [baseurl] - URL path without subdir.
      [error] - set to false. 
       */
      $upload_dir = wp_upload_dir();
      
      $url = $matches[0];
      $filename = $matches[1];
      
      $retUrl = $upload_dir['baseurl'] . "/baiduhi/$filename";
      $savePath = $upload_dir['basedir'] . "/baiduhi/$filename";
      
      if ( file_exists( $filename ) )
        return $retUrl;
      
      $destFp = fopen( $savePath, 'w+' );
      
      try 
      {
        $importer->httpDownload( $url, $destFp );
      }
      catch ( Exception $e )
      {
      }
      
      fclose( $destFp );
      
      if ( $e )
        throw new $e;
      
      return $retUrl;
    };
  }
  
  public function run()
  {
    $upload_dir = wp_upload_dir();
    $filename = $upload_dir['basedir'] . '/' . 'baiduhi';

    if ( !file_exists( "articles" ) && !mkdir( "articles" ) )
      throw new Exception( "Create articles dir faild." );
    
    if ( !file_exists( "recent" ) && !mkdir( "recent" ) )
      throw new Exception( "Create articles dir faild." );
      
    if ( !file_exists( $filename ) && !mkdir( $filename ) )
      throw new Exception( "Create attachment dir failed!" );
      
    $this->buildArticleFiles();
    $this->downloadArticles();
    $this->import();
  }
  
  public function import( )
  {
    global $wpdb;

//    $this->buildArticleFiles();
    
    $dir = dir("articles");
    
    while ( $entry = $dir->read() )
    {
      if ( $entry{0} == '.' )
        continue;

      $url = 'http://hi.baidu.com/meshikar/blog/item/' . $entry;

      $exists = $wpdb->query( "SELECT post_id FROM $wpdb->postmeta WHERE meta_key='source_url' AND meta_value='$url'" );

      if ( $exists )
        continue;
      
      try {

        echo "URL: ", $url, "\n";

        if ( filesize( "$dir->path/$entry" ) > 0 ) 
        {
          $html = file_get_contents( "$dir->path/$entry" );
        }
        else
        {
          $html = $this->httpGet( $url );
          file_put_contents( "$dir->path/$entry", $html );
        }
        
        $html = iconv( "GBK", "UTF-8", $html );

        $article = $this->parseArticlePage( $html );

        $catId = $this->getCatId( $article['category'] );

        $wp_post = array(
          'post_title' => $article['title'],
          'post_content' => $article['content'],
          'post_status' => 'publish',
          'post_author' => 1,
          'post_category' => array( $catId )
        );

        // Insert the post into the database
        $post_id = wp_insert_post( $wp_post );
        $where = array( 'ID' => $post_id );

        $wpdb->update( $wpdb->posts, array( 'post_date' => $article['post_date'],'post_date_gmt' => $article['post_date'] ), $where );

        add_post_meta( $post_id, 'source_url', $url );

        echo "OK: $article[title] -> [$post_id]\n";

        rename( "articles/$entry", "recent/$entry" );
      } 
      catch ( Exception $e )
      {
        echo "ERROR: [$url] " . $e->getMessage() . "\n";
      }
    }
  }
  
  public function parseArticlePage( $html )
  {
    $article = array( 
      'title' => '',
      'content' => '',
      'post_date' => '',
      'category' => ''
    );
    
    $startTag = '<div id=m_blog class=modbox style="overflow-x:hidden;">';
    $endTag = '<span class=x-blog-opt-sp>';
    
    $startPos = strpos( $html, $startTag ) + strlen( $startTag );
    $endPos = strpos( $html, $endTag );
    
    $blogContent = substr( $html, $startPos, $endPos );
    
    unset( $html );
    
    preg_match( '#<div class=tit>(.*)</div><div class=date>(\d{4}-\d{2}-\d{2} \d{1,2}:\d{1,2})</div>#Ui', $blogContent, $matches );
    $article['title'] = $matches[1];
    $article['post_date'] = date( 'Y-m-d H:i:s', strtotime( $matches[2] ) );;
    
    preg_match( '#<div id=blog_text class=cnt>(.*)</div></table><br><div class=opt id=blogOpt>#Uis' , $blogContent, $matches );
    
    $article['content'] = preg_replace_callback( '#http://hiphotos.baidu.com/meshikar/pic/item/(.*\.jpg)#Ui', $this->imageUrlProcessor, $matches[1] );
    
    preg_match( '#title="查看该分类中所有文章">类别：(.*)</a><span#Ui', $blogContent, $matches );
    
    $article['category'] = $matches[1];
    
    return $article;
  }
  
  public function buildArticleFiles( )
  {
    try 
    {
      $html = $this->httpGet( "http://hi.baidu.com/{$this->baiduAccount}/blog" );

      preg_match('#<div id=page>.*</div>#Uis', $html, $matches);

      preg_match_all( "#/{$this->baiduAccount}/blog/index/(\d+)#", $matches[0], $matches );

      rsort($matches[1]);
      $pageCount = $matches[1][0];
      

      for ( $page = 1; $page <= $pageCount; $page ++ )
      {
        if ( $page > 1 )
          $html = $this->httpGet( "http://hi.baidu.com/meshikar/blog/index/$page" );
        
        preg_match_all( "#/{$this->baiduAccount}/blog/item/(.+\.html)#Ui", $html, $matches );

        $articleUrls = array_unique( $matches[1] );
        
        foreach ( $articleUrls as $filename )
        {
          touch( "articles/$filename" );
          echo "Touched articles/$filename   [$page]\n";
        }
        
        sleep(0.5);
      }

    } 
    catch( Exception $e ) 
    {
      die( $e->getMessage() );
    }    
  }
  
  function downloadArticles()
  {
    $dir = dir("articles");
    
    while ( $entry = $dir->read() )
    {
      if ( $entry{0} == '.' )
        continue;
      
      $url = 'http://hi.baidu.com/meshikar/blog/item/' . $entry;
      $filename = $dir->path . '/' . $entry;
      $filesize = filesize( $filename );
      
      if ( $filesize != 0 )
        continue;
      
      $destFp = fopen($filename, 'w');
      
      unset( $exc );

      try
      {
        $this->httpDownload($url, $destFp);
        
        echo "OK: $url\n";
      }
      catch ( Exception $exc )
      {
      }
      
      fclose( $destFp );
      
      if ( $exc )
        throw $exc;
      
      sleep( 0.5 );
    }
  }
  
  public function httpDownload( $url, $destFp )
  {
    $ch = curl_init();
    curl_setopt( $ch, CURLOPT_URL, $url );
    curl_setopt( $ch, CURLOPT_FILE, $destFp );
    curl_setopt( $ch, CURLOPT_COOKIE, $this->cookie );

    $ret = curl_exec( $ch );
    $error = curl_error( $ch );
    
    curl_close($ch);
    unset( $ch );

    if ( !empty( $error ) )
      throw new Exception( $error );

    return $ret;
  }

  public function httpGet( $url )
  {
    $ch = curl_init();
    
    curl_setopt( $ch, CURLOPT_URL, $url );
    curl_setopt( $ch, CURLOPT_RETURNTRANSFER, true );
    curl_setopt( $ch, CURLOPT_COOKIE, $this->cookie );

    $ret = curl_exec( $ch );
    $error = curl_error( $ch );
    
    curl_close($ch);
    unset( $ch );

    if ( !empty( $error ) )
      throw new Exception( $error );

    return $ret;
  }
  
  public function getCatId( $cat_name )
  {
    $catId = 1;
    
    if ( $cat_name != '默认分类' )
    {
      $catId = get_cat_ID( $cat_name );

      if ( !$catId )
        $catId = wp_create_category( $cat_name );
    }
    
    return $catId;
  }
}


$cookie = 'YOUR COOKIE';

$importer = new BaiduHiToWrodpress( 'meshikar', $cookie );
$importer->run();
?>
